@@ -79,18 +79,17 @@ dic2.popitem
 #? int()
 dic2['asdf']
 
-# -----------------
-# set
-# -----------------
-set_t = {1,2}
+d = {'a': 3, 1.0: list}
 
-#? ['clear', 'copy']
-set_t.c
+#? int() list
+d.values()[0]
+##? int() list
+dict(d).values()[0]
 
-set_t2 = set()
-
-#? ['clear', 'copy']
-set_t2.c
+#? str()
+d.items()[0][0]
+#? int()
+d.items()[0][1]
 
 # -----------------
 # tuples
@@ -113,3 +112,16 @@ tup3.index
 tup4 = 1,""
 #? ['index']
 tup4.index
+
+# -----------------
+# set
+# -----------------
+set_t = {1,2}
+
+#? ['clear', 'copy']
+set_t.c
+
+set_t2 = set()
+
+#? ['clear', 'copy']
+set_t2.c
