@@ -1,44 +1,29 @@
 from textwrap import dedent
 
-import parso
-
-from jedi._compatibility import builtins, is_py3
 from jedi.evaluate import compiled
 from jedi.evaluate.context import instance
-from jedi.evaluate.context.function import FunctionContext
-from jedi.evaluate import Evaluator
-from jedi.evaluate.project import Project
-from jedi.parser_utils import clean_scope_docstring
-from jedi import Script
 
 
-def _evaluator():
-    return Evaluator(parso.load_grammar(), Project())
-
-
-def test_simple():
-    e = _evaluator()
-    bltn = compiled.CompiledObject(e, builtins)
-    obj = compiled.CompiledObject(e, '_str_', bltn)
-    upper, = obj.py__getattribute__('upper')
+def test_simple(evaluator):
+    obj = compiled.create_simple_object(evaluator, u'_str_')
+    upper, = obj.py__getattribute__(u'upper')
     objs = list(upper.execute_evaluated())
     assert len(objs) == 1
     assert isinstance(objs[0], instance.CompiledInstance)
 
 
-def test_fake_loading():
-    e = _evaluator()
-    assert isinstance(compiled.create(e, next), FunctionContext)
-
-    builtin = compiled.get_special_object(e, 'BUILTINS')
-    string, = builtin.py__getattribute__('str')
-    from_name = compiled._create_from_name(e, builtin, string, '__init__')
-    assert isinstance(from_name, FunctionContext)
+def test_fake_loading(evaluator):
+    builtin = compiled.get_special_object(evaluator, u'BUILTINS')
+    string, = builtin.py__getattribute__(u'str')
+    from_name = compiled.context.create_from_name(evaluator, string, u'__init__')
+    assert from_name.tree_node
 
 
-def test_fake_docstr():
-    node = compiled.create(_evaluator(), next).tree_node
-    assert clean_scope_docstring(node) == next.__doc__
+def test_fake_docstr(evaluator):
+    next_ = compiled.builtin_from_name(evaluator, u'next')
+    assert next_.py__doc__()
+    assert next_.tree_node is not None
+    assert next_.py__doc__() == next.__doc__
 
 
 def test_parse_function_doc_illegal_docstr():
@@ -47,26 +32,28 @@ def test_parse_function_doc_illegal_docstr():
 
     doesn't have a closing bracket.
     """
-    assert ('', '') == compiled._parse_function_doc(docstr)
+    assert ('', '') == compiled.context._parse_function_doc(docstr)
 
 
-def test_doc():
+def test_doc(evaluator):
     """
     Even CompiledObject docs always return empty docstrings - not None, that's
     just a Jedi API definition.
     """
-    obj = compiled.CompiledObject(_evaluator(), ''.__getnewargs__)
+    str_ = compiled.create_simple_object(evaluator, u'')
+    # Equals `''.__getnewargs__`
+    obj = compiled.create_from_name(evaluator, str_, u'__getnewargs__')
     assert obj.py__doc__() == ''
 
 
-def test_string_literals():
+def test_string_literals(Script, environment):
     def typ(string):
         d = Script("a = %s; a" % string).goto_definitions()[0]
         return d.name
 
     assert typ('""') == 'str'
     assert typ('r""') == 'str'
-    if is_py3:
+    if environment.version_info.major > 2:
         assert typ('br""') == 'bytes'
         assert typ('b""') == 'bytes'
         assert typ('u""') == 'str'
@@ -75,7 +62,7 @@ def test_string_literals():
         assert typ('u""') == 'unicode'
 
 
-def test_method_completion():
+def test_method_completion(Script, environment):
     code = dedent('''
     class Foo:
         def bar(self):
@@ -83,18 +70,18 @@ def test_method_completion():
 
     foo = Foo()
     foo.bar.__func__''')
-    if is_py3:
+    if environment.version_info.major > 2:
         result = []
     else:
         result = ['__func__']
     assert [c.name for c in Script(code).completions()] == result
 
 
-def test_time_docstring():
+def test_time_docstring(Script):
     import time
     comp, = Script('import time\ntime.sleep').completions()
     assert comp.docstring() == time.sleep.__doc__
 
 
-def test_dict_values():
+def test_dict_values(Script):
     assert Script('import sys\nsys.modules["alshdb;lasdhf"]').goto_definitions()
